/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2002, 2004                                        */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

package com.ibm.staf.service.stax;
import com.ibm.staf.*;
import com.ibm.staf.STAFUtil;
import org.python.core.Py;
import org.python.core.PyList;
import java.util.*;

public class STAXProcessAction implements STAXAction,
                                          STAXSTAFRequestCompleteListener,
                                          STAXProcessCompleteListener,
                                          STAXThreadCompleteListener,
                                          STAXTimedEventListener
{
    static boolean debug = false;

    static final int INIT = 0;
    static final int WAIT_REQUEST = 1;
    static final int REQUEST_ERROR = 2;
    static final int REQUEST_TIMEOUT = 3;
    static final int WAIT_PROCESS = 4;
    static final int PROCESS_COMPLETE_WAIT_ACTION = 5;
    static final int PROCESS_COMPLETE = 6;
    static final int COMPLETE = 7;

    static final String INIT_STRING = "INIT";
    static final String WAIT_REQUEST_STRING = "WAIT_REQUEST";
    static final String REQUEST_ERROR_STRING = "REQUEST_ERROR";
    static final String REQUEST_TIMEOUT_STRING = "REQUEST_TIMEOUT";
    static final String WAIT_PROCESS_STRING = "WAIT_PROCESS";
    static final String PROCESS_COMPLETE_WAIT_SEQUENCE_STRING = 
        "PROCESS_COMPLETE_WAIT_ACTION";
    static final String PROCESS_COMPLETE_STRING = "PROCESS_COMPLETE";
    static final String COMPLETE_STRING = "COMPLETE";
    static final String STATE_UNKNOWN_STRING = "UNKNOWN";

    public STAXProcessAction()
    { /* Do Nothing */ }

    public STAXProcessActionFactory getActionFactory() { return fFactory; }
    public void setActionFactory(STAXProcessActionFactory factory) 
    { 
        fFactory = factory; 
    }

    public String getLocation() { return fLocation; }
    public void   setLocation(String location) 
    { 
        fLocation = location; 
        fUnevalLocation = location;
   }

    public String getCommand() { return fCommand; }
    public void   setCommand(String command) 
    { 
        fCommand = command; 
        fUnevalCommand = command;
    }
    
    public String getName() { return fName; }
    public void   setName(String name)
    { 
        fName = name; 
        fUnevalName = name;
    }
    
    public String getWorkload() { return fWorkload; }
    public void   setWorkload(String workload, String workloadIf) 
    { 
        fWorkload = workload; 
        fUnevalWorkload = workload;
        fWorkloadIf = workloadIf;
    }
    
    public String getTitle() { return fTitle; }
    public void   setTitle(String title, String titleIf) 
    { 
        fTitle = title; 
        fUnevalTitle = title;
        fTitleIf = titleIf;
    }
    
    public String getParms() { return fParms; }
    public void   setParms(String parms, String parmsIf) 
    { 
        fParms = parms;
        fUnevalParms = parms;
        fParmsIf = parmsIf;
    }
    
    public String getWorkdir() { return fWorkdir; }
    public void   setWorkdir(String workdir, String workdirIf) 
    { 
        fWorkdir = workdir;
        fUnevalWorkdir = workdir;
        fWorkdirIf = workdirIf;
    }
    
    public Vector getVars() { return fVars; }
    public void   setVars(String vars, String varsIf) 
    { 
        fUnevalVars.add(vars);
        fVarsIf.add(varsIf);
    }
    
    public Vector getEnvs() { return fEnvs; }
    public void   setEnvs(String envs, String envsIf) 
    { 
        fUnevalEnvs.add(envs);
        fEnvsIf.add(envsIf);
    }
    
    public String getUseprocessvars() { return fUseprocessvars; }
    public void   setUseprocessvars(String useprocessvars,
                                    String useprocessvarsIf) 
    { 
        fUseprocessvars = useprocessvars;
        fUnevalUseprocessvars = useprocessvars; 
        fUseprocessvarsIf = useprocessvarsIf;
    }
        
    public String getStopusing() { return fStopusing; }
    public void   setStopusing(String stopusing, String stopusingIf) 
    { 
        fStopusing = stopusing;
        fUnevalStopusing = stopusing; 
        fStopusingIf = stopusingIf;
    }
    
    public String getConsole() { return fConsole; }
    public void   setConsole(String console, String consoleIf) 
    { 
        fConsole = console; 
        fUnevalConsole = console;
        fConsoleIf = consoleIf;
    }

    public String getFocus() { return fFocus; }
    public void   setFocus(String focus, String focusIf) 
    { 
        fFocus = focus; 
        fUnevalFocus = focus;
        fFocusIf = focusIf;
    }
        
    public String getUsername() { return fUsername; }
    public void   setUsername(String username, String usernameIf) 
    { 
        fUsername = username;
        fUnevalUsername = username; 
        fUsernameIf = usernameIf;
    }
    
    public String getPassword() { return fPassword; }
    public void   setPassword(String password, String passwordIf) 
    { 
        fPassword = password; 
        fUnevalPassword = password;
        fPasswordIf = passwordIf;
    }
    
    public String getDisabledauth() { return fDisabledauth; }
    public void   setDisabledauth(String disabledauth, String disabledauthIf) 
    { 
        fDisabledauth = disabledauth; 
        fUnevalDisabledauth = disabledauth;
        fDisabledauthIf = disabledauthIf;
    }
    
    public String getStatichandlename() { return fStatichandlename; }
    public void   setStatichandlename(String statichandlename, 
                                      String statichandlenameIf) 
    { 
        fStatichandlename = statichandlename; 
        fUnevalStatichandlename = statichandlename;
        fStatichandlenameIf = statichandlenameIf;
    }
        
    public String getStdin() { return fStdin; }
    public void   setStdin(String stdin, String stdinIf) 
    { 
        fStdin = stdin; 
        fUnevalStdin = stdin;
        fStdinIf = stdinIf;
    }
    
    public String getStdout() { return fStdout; }
    public String getStdoutMode() { return fStdoutMode; }
    public void   setStdout(String stdout, String stdoutMode, String stdoutIf) 
    { 
        fStdout = stdout; 
        fUnevalStdout = stdout;
        fStdoutIf = stdoutIf;
        fStdoutMode = stdoutMode;
        fUnevalStdoutMode = stdoutMode;
    }
        
    public String getStderr() { return fStderr; }
    public String getStderrMode() { return fStderrMode; }

    public void   setStderr(String stderr, String stderrMode, String stderrIf) 
    { 
        fStderr = stderr;
        fUnevalStderr = stderr; 
        fStderrIf = stderrIf;
        fStderrMode = stderrMode;
        fUnevalStderrMode = stderrMode;
    }

    public String getReturnStdout() { return fReturnStdout; }
    public void   setReturnStdout(String returnStdout, String returnStdoutIf) 
    {
        fReturnStdout = returnStdout;
        fUnevalReturnStdout = returnStdout;
        fReturnStdoutIf = returnStdoutIf;
    }
    
    public String getReturnStderr() { return fReturnStderr; }
    public void   setReturnStderr(String returnStderr, String returnStderrIf) 
    {
        fReturnStderr = returnStderr;
        fUnevalReturnStderr = returnStderr;
        fReturnStderrIf = returnStderrIf;
    }

    public Vector getReturnFiles() { return fReturnFiles; }
    public void   setReturnFiles(String returnFile, String returnFileIf) 
    { 
        fUnevalReturnFiles.add(returnFile);
        fReturnFilesIf.add(returnFileIf);
    }
    
    public String getOther() { return fOther; }
    public void   setOther(String other, String otherIf) 
    { 
        fOther = other; 
        fUnevalOther = other;
        fOtherIf = otherIf;
    }
    
    public STAXAction getProcessAction() { return fProcessAction; }
    public void   setProcessAction(STAXAction processAction, 
        String processActionIf) 
    { 
        fProcessAction = processAction;
        fProcessActionIf = processActionIf;
        fUnevalProcessActionIf = processActionIf;
    }
    
    public String getCommandMode() { return fCommandMode; }
    public void   setCommandMode(String mode) 
    {
        fCommandMode = mode;
        fUnevalCommandMode = mode;
    }

    public String getCommandShell() { return fCommandShell; }
    public void   setCommandShell(String shell) 
    {
        fCommandShell = shell;
        fUnevalCommandShell = shell;
    }

    public STAXThread getThread() { return fThread; }

    public String getCurrentBlockName() { return fCurrentBlockName; }

    public STAXTimestamp getStartTimestamp() { return fStartTimestamp; }

    public int getProcessHandle() { return fProcessHandle; }

    public String getStateAsString()
    {
        switch (fState)
        {
            case INIT:
                return INIT_STRING;
            case WAIT_REQUEST:
                return WAIT_REQUEST_STRING;
            case REQUEST_ERROR:
                return REQUEST_ERROR_STRING;
            case REQUEST_TIMEOUT:
                return REQUEST_TIMEOUT_STRING;
            case WAIT_PROCESS:
                return WAIT_PROCESS_STRING;
            case PROCESS_COMPLETE:
                return PROCESS_COMPLETE_STRING;
            case COMPLETE:
                return COMPLETE_STRING;
            case PROCESS_COMPLETE_WAIT_ACTION: 
                return PROCESS_COMPLETE_WAIT_SEQUENCE_STRING;
            default:
                return STATE_UNKNOWN_STRING;
        }
    }
    
    public String getInfo()
    {
        return fName;
    }

    public String getDetails()
    {
        String result = 
               "State:" + getStateAsString() +
               ";Name:" + fName + ";Location:" + fLocation +
               ";Command:" + fCommand;
        if (!fCommandMode.equals(""))
            result += ";CommandMode:" + fCommandMode;
        if (!fCommandShell.equals(""))
            result += ";CommandShell:" + fCommandShell;
        if (!fWorkload.equals(""))
            result += ";Workload:" + fWorkload;
        if (!fTitle.equals(""))
            result += ";Title:" + fTitle;
        if (!fParms.equals(""))
            result += ";Parms:" + fParms;
        if (!fWorkdir.equals(""))
            result += ";Workdir:" + fWorkdir;

        for (int i = 0; i < fVars.size(); i++)
        {
            result += ";Var" + (i + 1) + ":" + (String)fVars.elementAt(i);
        }

        for (int i = 0; i < fEnvs.size(); i++)
        {
            result += ";Env" + (i + 1) + ":" + (String)fEnvs.elementAt(i);
        }

        if (!fUseprocessvars.equals(""))
            result += ";Useprocessvars:" + fUseprocessvars;
        if (!fStopusing.equals(""))
            result += ";Stopusing:" + fStopusing;
        if (!fConsole.equals(""))
            result += ";Console:" + fConsole;
        if (!fFocus.equals(""))
            result += ";Focus:" + fFocus;
        if (!fUsername.equals(""))
            result += ";Username:" + fUsername;
        if (!fPassword.equals(""))
            result += ";Password:" + "*******";
        if (!fDisabledauth.equals(""))
            result += ";Disabledauth:" + fDisabledauth;
        if (!fStatichandlename.equals(""))
            result += ";Statichandlename:" + fStatichandlename;
        if (!fStdin.equals(""))
            result += ";Stdin:" + fStdin;
        if (!fStdout.equals(""))
            result += ";Stdout:" + fStdout;
        if (!fStdoutMode.equals(""))
            result += ";StdoutMode:" + fStdoutMode;
        if (!fStderr.equals(""))
            result += ";Stderr:" + fStderr;
        if (!fStderrMode.equals(""))
            result += ";StderrMode:" + fStderrMode;
        if (!fReturnStdout.equals(""))
            result += ";ReturnStdout:" + fReturnStdout;
        if (!fReturnStderr.equals(""))
            result += ";ReturnStderr:" + fReturnStderr;
        
        for (int i = 0; i < fReturnFiles.size(); i++)
        {
            result += ";ReturnFile" + (i + 1) + ":" +
                      (String)fReturnFiles.elementAt(i);
        }

        if (!fOther.equals(""))
            result += ";Other:" + fOther;

        result += ";HoldThreadCondition:" + fHoldCondition +
                  ";RequestNumber:" + fRequestNumber +
                  ";RequestRC:" + fRequestRC +
                  ";Request:" + fRequest +
                  ";RequestResult:" + fRequestResult +
                  ";ProcessHandle:" + fProcessHandle +
                  ";ProcessRC:" + fProcessRC +
                  ";ProcessTimestamp:" + fProcessTimestamp +
                  ";StartTimestamp:" + fStartTimestamp +
                  ";BlockName:" + fCurrentBlockName;

        return result;
    }

    public void execute(STAXThread thread)
    {
        synchronized (this)
        {
            if (fState == INIT)
            {            
                fThread = thread;
 
                // Evaluate the process element and attribute values and
                // generate a process start request and assign to fRequest.

                if (generateProcessStartRequest() != 0) return;
            
                // Set current block name
                try
                {
                    fCurrentBlockName = fThread.pyStringEval(
                        "STAXCurrentBlock");
                }
                catch (STAXPythonEvaluationException e)
                {
                    fCurrentBlockName = "";  //Shouldn't happen
                }

                // Set to the current date and time.
                fStartTimestamp = new STAXTimestamp();
         
                fState = WAIT_REQUEST;
                
                fProcessKey = new Integer(
                    fThread.getJob().getNextProcessKey()).toString();
                
                // Submit PROCESS START STAF Command

                STAFResult submitResult = fThread.getJob().submitAsync(
                    fLocation, "process", "START NOTIFY ONEND KEY " +
                    fProcessKey + " " + fRequest, this);
                    
                if (submitResult.rc == 0 )
                {
                    fRequestNumber = (new Integer(submitResult.result)).
                                                  intValue();
                }
                
                if (fRequestNumber == 0)
                {
                    // Request failed - Raise a STAXProcessStartError signal
                    fState = COMPLETE;
                    fThread.popAction();
                    
                    String msg = "<process>\n" +
                        "\n  Name      : " + fName +
                        "\n  Location  : " + fLocation +
                        "\n  Service   : " + "PROCESS" +
                        "\n  Request   : " + "START " + fRequest +
                        "\n  RC        : " + submitResult.rc +
                        "\n  STAFResult: " + submitResult.result;

                    fThread.setSignalMsgVar("STAXProcessStartErrorMsg", msg);
                    fThread.raiseSignal("STAXProcessStartError");
                    
                    return;        
                }
                
                if (debug)
                {
                    System.out.println("STAXProcessAction::execute(): " +
                        "Add timed event - timeout=" +
                        fThread.getJob().getSTAX().getProcessTimeout());
                }
            
                // Add a TimedEvent to the queue and return 

                fTimedEvent = new STAXTimedEvent(System.currentTimeMillis() + 
                    fThread.getJob().getSTAX().getProcessTimeout(), this);

                fThread.getJob().getSTAX().getTimedEventQueue().addTimedEvent(
                    fTimedEvent);

                fThread.addCondition(fHoldCondition);
            }
            else if (fState == REQUEST_ERROR)
            {
                fState = COMPLETE;
                fThread.popAction();

                if (debug)
                {
                    System.out.println("STAXProcessAction::execute(): " +
                        "fState=REQUEST_ERROR - Raise STAXProcessStartError" +
                        " signal");
                }

                String msg = "<process>\n" +
                    "\n  Name      : " + fName +
                    "\n  Location  : " + fLocation +
                    "\n  Service   : " + "PROCESS" +
                    "\n  Request   : " + "START " + fRequest +
                    "\n  RC        : " + fRequestRC +
                    "\n  STAFResult: " + fRequestResult;

                fThread.setSignalMsgVar("STAXProcessStartErrorMsg", msg);
                fThread.raiseSignal("STAXProcessStartError");
            }
            else if (fState == REQUEST_TIMEOUT)
            {
                // The process request did not start within the timeout,
                // PROCESSTIMEOUT, (requestComplete was not called) so
                // generate a STAXProcessStartTimeout signal

                fState = COMPLETE;
                fThread.popAction();
         
                if (debug)
                {
                    System.out.println("STAXProcessAction::execute(): " +
                                       "fState=REQUEST_TIMEOUT - Raise " + 
                                       "STAXProcessStartTimeout signal");
                }

                String msg = "<process>\n" +
                    "\n  Name      : " + fName +
                    "\n  Location  : " + fLocation +
                    "\n  Service   : " + "PROCESS" +
                    "\n  Request   : " + "START " + fRequest +
                    "\n\n  Process did not start within timeout value " +
                    fThread.getJob().getSTAX().getProcessTimeout();

                fThread.setSignalMsgVar("STAXProcessStartTimeoutMsg", msg);
                fThread.raiseSignal("STAXProcessStartTimeout");

                // Set RC to indicate a STAXProcessStartTimeout signal
                fRequestRC = STAFResult.Timeout;
                fThread.pySetVar("RC", new Integer(STAFResult.Timeout));
            }
            else if (fState == PROCESS_COMPLETE)
            {
                fState = COMPLETE;
                fThread.popAction();
            }
            else if (fState == COMPLETE)
            {
                // Note: We shouldn't be called in this state.
                fThread.popAction();
            }
        }  // End synchronized (this)
    }


    // Note that this entire method is synchronized
    public synchronized void handleCondition(STAXThread thread, 
        STAXCondition cond)
    {
        synchronized (this)
        {
            if (debug)
            {
                System.out.println("STAXProcessAction::handleCondition(): " +
                                   "fState=" + getStateAsString());
            }
                
            if ((fState == WAIT_PROCESS) || 
                (fState == PROCESS_COMPLETE_WAIT_ACTION))
            {
                // Set this first, in case message comes in before setting
                // state below

                fState = COMPLETE;

                if (debug)
                {
                    System.out.println(
                        "STAXProcessAction::handleCondition(): " +
                        "removeHoldCondition and stop handle");
                }
                thread.removeCondition(fHoldCondition);
            
                thread.getJob().submitAsyncForget(fLocation, "process",
                    "stop handle " + fProcessHandle);

                // Generate a stop process event
                generateProcessStopEvent();

                // Remove the process from the processRequestMap 
                String key = fLocation + ":" + fProcessHandle;
                TreeMap processes = (TreeMap)thread.getJob().
                                        getData("processRequestMap");
                synchronized (processes)
                {
                    processes.remove(key.toLowerCase());
                }
            }

            fState = COMPLETE;
            thread.popAction();
        }
    }

    public STAXAction cloneAction()
    {
        STAXProcessAction clone = new STAXProcessAction();
        
        clone.fUnevalLocation = fUnevalLocation;
        clone.fUnevalCommand = fUnevalCommand;
        clone.fUnevalName = fUnevalName;
        clone.fUnevalWorkload = fUnevalWorkload;
        clone.fUnevalWorkdir = fUnevalWorkdir;
        clone.fUnevalTitle = fUnevalTitle;
        clone.fUnevalParms = fUnevalParms;
        clone.fUnevalWorkdir = fUnevalWorkdir;
        clone.fUnevalVars = fUnevalVars;
        clone.fUnevalEnvs = fUnevalEnvs;
        clone.fUnevalUseprocessvars = fUnevalUseprocessvars;
        clone.fUnevalStopusing = fUnevalStopusing;
        clone.fUnevalConsole = fUnevalConsole;
        clone.fUnevalFocus = fUnevalFocus;
        clone.fUnevalUsername = fUnevalUsername;
        clone.fUnevalPassword = fUnevalPassword;
        clone.fUnevalDisabledauth = fUnevalDisabledauth;
        clone.fUnevalStatichandlename = fUnevalStatichandlename;
        clone.fUnevalStdin = fUnevalStdin;
        clone.fUnevalStdout = fUnevalStdout;
        clone.fUnevalStdoutMode = fUnevalStdoutMode;
        clone.fUnevalStderr = fUnevalStderr;
        clone.fUnevalStderrMode = fUnevalStderrMode;
        clone.fUnevalReturnStdout = fUnevalReturnStdout;
        clone.fUnevalReturnStderr = fUnevalReturnStderr;
        clone.fUnevalReturnFiles = fUnevalReturnFiles;
        clone.fUnevalOther = fUnevalOther;
        clone.fUnevalCommandMode = fUnevalCommandMode;
        clone.fUnevalCommandShell = fUnevalCommandShell;

        clone.fCurrentBlockName = fCurrentBlockName;
        clone.fFactory = fFactory;
        clone.fLocation = fLocation;
        clone.fCommand = fCommand;
        clone.fName = fName;
        clone.fWorkload = fWorkload;
        clone.fWorkloadIf = fWorkloadIf;
        clone.fWorkdir = fWorkdir;
        clone.fWorkdirIf = fWorkdirIf;
        clone.fTitle = fTitle;
        clone.fTitleIf = fTitleIf;
        clone.fParms = fParms;
        clone.fParmsIf = fParmsIf;
        clone.fVars = new Vector();
        clone.fVarsIf = fVarsIf;
        clone.fEnvs = new Vector();
        clone.fEnvsIf = fEnvsIf;
        clone.fUseprocessvars = fUseprocessvars;
        clone.fUseprocessvarsIf = fUseprocessvarsIf;
        clone.fStopusing = fStopusing;
        clone.fStopusingIf = fStopusingIf;
        clone.fConsole = fConsole;
        clone.fConsoleIf = fConsoleIf;
        clone.fFocus = fFocus;
        clone.fFocusIf = fFocusIf;
        clone.fUsername = fUsername;
        clone.fUsernameIf = fUsernameIf;
        clone.fPassword = fPassword;
        clone.fPasswordIf = fPasswordIf;
        clone.fDisabledauth = fDisabledauth;
        clone.fDisabledauthIf = fDisabledauthIf;
        clone.fStatichandlename = fStatichandlename;
        clone.fStatichandlenameIf = fStatichandlenameIf;
        clone.fStdin = fStdin;
        clone.fStdinIf = fStdinIf;
        clone.fStdout = fStdout;
        clone.fStdoutIf = fStdoutIf;
        clone.fStdoutMode = fStdoutMode;
        clone.fStderr = fStderr;
        clone.fStderrIf = fStderrIf;
        clone.fStderrMode = fStderrMode;
        clone.fReturnStdout = fReturnStdout;
        clone.fReturnStdoutIf = fReturnStdoutIf;
        clone.fReturnStderr = fReturnStderr;
        clone.fReturnStderrIf = fReturnStderrIf;
        clone.fReturnFiles = new Vector();
        clone.fReturnFilesIf = fReturnFilesIf;
        clone.fOther = fOther;
        clone.fOtherIf = fOtherIf;
        clone.fProcessAction = fProcessAction;
        clone.fProcessActionIf = fProcessActionIf;
        clone.fCommandMode = fCommandMode;
        clone.fCommandShell = fCommandShell;
       
        return clone;
    }

    // STAXTimedEventListener method

    // Note that this entire method is synchronized
    public synchronized void timedEventOccurred(STAXTimedEvent timedEvent)
    {
        if (debug)
        {
            System.out.println(
                "STAXProcessAction::timedEventOccurred(): fState=" +
                getStateAsString());
        }
            
        if (fState == WAIT_REQUEST)
        {
            if (debug)
            {
                System.out.println(
                    "STAXProcessAction::timedEventOccurred(): Set " +
                    "fState=REQUEST_TIMEOUT, remove holdCondition, " +
                    "and schedule thread");
            }                                                     

            fState = REQUEST_TIMEOUT;
            fThread.removeCondition(fHoldCondition);
            fThread.schedule();
        }
    }

    // STAXSTAFRequestCompleteListener method

    // Note that this entire method is synchronized
    public synchronized void requestComplete(int requestNumber,
        STAFResult result)
    {
        // Already COMPLETE, so something like a STAXProcessRequestTimeout
        // signal could have already been raised, so just return

        if (fState == COMPLETE) return;

        // Remove request timeout event 

        if (debug)
        {
            System.out.println("STAXProcessAction::requestComplete(): " +
                               "removeTimedEvent()");
        }
        fThread.getJob().getSTAX().getTimedEventQueue().removeTimedEvent(
            fTimedEvent);

        fThread.pySetVar("RC", new Integer(result.rc));
        
        if (result.rc == 0)
        {
            boolean processComplete = false;

            try
            {
                // XXX: Why do we store the handle in "int" form instead of
                //      just storing the handle in "string" form

                fProcessHandle = Integer.parseInt(result.result);
                fState = WAIT_PROCESS;

                // Register for Process Completion

                String key = new String();
                String machine = fLocation;
                
                if (fUsingProcessKeyOption)
                {
                    key = fProcessKey;
                }
                else
                {
                    key = machine + "/" + fProcessHandle;
                }

                HashMap processMap =
                        (HashMap)fThread.getJob().getData("processMap");

                synchronized (processMap)
                {
                    // If we have a race condition, we may have already
                    // received the "process end" message
                            
                    if (processMap.containsKey(key))
                    {
                        if (debug)
                        {
                            System.out.println(
                                "STAXProcessAction::requestComplete(): " + 
                                "Race condition - remove from processMap");
                        }

                        STAXSTAFMessage completionMessage = 
                            (STAXSTAFMessage)processMap.get(key);
                                
                        processMap.remove(key);
                            
                        processComplete(machine, fProcessHandle,
                            completionMessage.getProcessRC(), 
                            completionMessage.getProcessResultAsList(),
                            completionMessage.getProcessTimestamp());

                        processComplete = true;
                    }
                    else
                    {
                        if (debug)
                        {
                            System.out.println(
                                "STAXProcessAction::requestComplete(): " +
                                "Not race condition: add to processMap");
                        }

                        processMap.put(key.toLowerCase(), this);
                    }
                }
            }
            catch (NumberFormatException e)
            {
                // XXX: Do something.  Maybe set fState = ERROR?
            }

            if (!processComplete)
            {
                // Add the process to the processRequestMap 
                String key = fLocation + ":" + fProcessHandle;
                TreeMap processes = (TreeMap)fThread.getJob().
                                    getData("processRequestMap");
                synchronized (processes)
                {
                    if (debug)
                    {
                        System.out.println(
                            "STAXProcessAction::requestComplete(): " +
                            "Add to processRequestMap");
                    }

                    processes.put(key.toLowerCase(), this);
                }

                // Generate a process start event
                generateProcessStartEvent();
            }
        
            synchronized (fThreadMap)
            {
                if ((fProcessAction != null) && fProcessActionFlag)
                {
                    fThread.pySetVar("STAXProcessHandle", 
                        String.valueOf(fProcessHandle));
                      
                    STAXThread childThread = fThread.createChildThread();
                     
                    fThreadMap.put(new 
                        Integer(childThread.getThreadNumber()), 
                        childThread);
                    
                    STAXAction childThreadAction = fProcessAction;
                   
                    childThread.addCompletionNotifiee(this);
                    childThread.pushAction(
                        childThreadAction.cloneAction());
                    childThread.schedule();
                }
            }
        }
        else
        {
            if ((result.rc == STAFResult.InvalidRequestString) &&
                fUsingProcessKeyOption)
            {
                // The target machine may not support the KEY option,
                // so re-submit the request without the KEY option.
                // If the second submit fails, then the InvalidRequestString
                // is not due to the KEY option not being supported.

                fUsingProcessKeyOption = false;
                
                // Submit PROCESS START STAF Command

                STAFResult submitResult = fThread.getJob().submitAsync(
                    fLocation, "process", "start notify onend " +
                    fRequest, this);
                    
                if (submitResult.rc == 0 )
                {
                    fRequestNumber = (new Integer(submitResult.result)).
                                                  intValue();
                }
                
                if (fRequestNumber == 0)
                {
                    // Request failed - Raise a STAXProcessStartError signal
                    fState = COMPLETE;
                    fThread.popAction();
                    
                    String msg = "<process>\n" +
                        "\n  Name      : " + fName +
                        "\n  Location  : " + fLocation +
                        "\n  Service   : " + "PROCESS" +
                        "\n  Request   : " + "START " + fRequest +
                        "\n  RC        : " + submitResult.rc +
                        "\n  STAFResult: " + submitResult.result;

                    fThread.setSignalMsgVar("STAXProcessStartErrorMsg", msg);
                    fThread.raiseSignal("STAXProcessStartError");

                    return;        
                }
            }
            else
            {
                fThread.pySetVar("STAFResult", result.result);
            
                fRequestRC = result.rc;
                fRequestResult = result.result;

                if (debug)
                {
                    System.out.println(
                        "STAXProcessAction::requestComplete(): " +
                        "set fState=REQUEST_ERROR and remove holdCondition " +
                        "and schedule thread");
                }

                fState = REQUEST_ERROR;
                fThread.removeCondition(fHoldCondition);
                fThread.schedule();
            }
        }
    }

    public synchronized void processComplete(String machine, int handle, 
        long rc, List fileList, String timestamp)
    {
        if (debug)
        {
            System.out.println("STAXProcessAction::processComplete(): " +
                               "fState=" + getStateAsString());
        }

        fThread.getJob().submitAsyncForget(fLocation, "process",
            "free handle " + fProcessHandle);

        if (fState == COMPLETE) return;
       
        // We only generate the stop event if we aren't COMPLETE.  If we
        // were terminated before the process finished, then we generated
        // the stop event at that time.
        generateProcessStopEvent();

        // Remove the process from the processRequestMap 
        String key = fLocation + ":" + fProcessHandle;
        TreeMap processes = (TreeMap)fThread.getJob().getData(
            "processRequestMap");

        synchronized (processes)
        {
            if (debug)
            {
                System.out.println("STAXProcessAction::processComplete(): " +
                                   "Remove from processRequestMap");
            }

            processes.remove(key.toLowerCase());
        }

        fThread.pySetVar("RC", new Long(rc));
        fProcessRC = rc;

        PyList resultList = getResultList(fileList);

        if (resultList != null)
        {          
            try
            {
                fThread.pySetVar("STAXResult", resultList);
            }
            catch (Exception ex)
            {
                // This should never happen
                ex.printStackTrace();

                String msg = "<process>\n" +
                    "\n  Details : " + this.getDetails() +
                    "\n  fileList: " + fileList +
                    "\n\n  Error setting STAXResult to resultList." +
                    "\n\n" + ex.toString();

                fThread.setSignalMsgVar("STAXPythonEvalMsg", msg);
                fThread.raiseSignal("STAXPythonEvaluationError");
            }
        }
           
        fProcessTimestamp = timestamp;

        if (fThreadMap.isEmpty())
        {
            if (debug)
            {
                System.out.println("STAXProcessAction::processComplete(): " +
                                   "remove condition and schedule thread");
            }

            fState = PROCESS_COMPLETE;
            fThread.removeCondition(fHoldCondition);
            fThread.schedule();
        }
        else
        {
            fState = PROCESS_COMPLETE_WAIT_ACTION;
        }
    }

    public synchronized void threadComplete(STAXThread thread, int endCode)
    {
        synchronized(fThreadMap)
        {
            fThreadMap.remove(new Integer(thread.getThreadNumber()));

            if (fThreadMap.isEmpty() && 
                (fState == PROCESS_COMPLETE_WAIT_ACTION))
            {
                if (debug)
                {
                    System.out.println(
                        "STAXProcessAction::threadComplete(): " +
                        "remove condition and schedule thread");
                }

                fState = PROCESS_COMPLETE;
                fThread.removeCondition(fHoldCondition);
                fThread.schedule();
            }
        }
    }
    
    public PyList getResultList(List fileList)
    {
        if (fileList == null || fileList.size() == 0) return null;

        // Create a Python List to contain a list of returned file info:
        //   [ [ File1 RC, File1 Data ], [ File2 RC, File2 Data ], ... ] 
        PyList resultList = new PyList();
        Iterator iter = fileList.iterator();

        try
        {
            while (iter.hasNext())
            {
                Map fileInfoMap = (Map)iter.next();

                // Create a Python List to contain file information (RC & data)
                PyList fileInfoList = new PyList();

                // Assign file RC
                fileInfoList.append(Py.java2py(
                    new Integer((String)fileInfoMap.get("rc"))));
                fileInfoList.append(Py.java2py(
                    (String)fileInfoMap.get("data")));

                // Add file information List to result List
                resultList.append(fileInfoList);
            }
        }
        catch (Exception ex)
        {
            // This should never happen
            ex.printStackTrace();

            String msg = "<process>\n" +
                "\n  Details  : " + this.getDetails() +
                "\n  File List: " + fileList.toString() +
                "\n\n  Error getting STAXResult from result. " +
                "Report to STAX Developers." +
                "\n\n" + ex.toString();

            fThread.setSignalMsgVar("STAXPythonEvalMsg", msg);
            fThread.raiseSignal("STAXPythonEvaluationError");
        }
        
        return resultList;
    }

    // This method evaluates the process element/sub-elements and attribute
    // values and generates a process start request and assigns it to fRequest.
    // If an error occurs, it raises the appropriate signal and returns a 
    //non-zero value.

    private int generateProcessStartRequest()
    {
        StringBuffer request = new StringBuffer("");
        String evalElem = "";
        String evalAttr = "";

        try
        {
            // Set RC initially so if an error occurs (that does not
            // terminate the job) before a process request is
            // submitted, <if expr="RC != 0"> won't fail.
            fThread.pySetVar("RC", new Integer(-1));

            // Set initial values for other variables set by process
            fThread.pySetVar("STAFResult", Py.None);
            fThread.pySetVar("STAXResult", Py.None);
            fThread.pySetVar("STAXProcessHandle", new Integer(0));

            evalElem = "location";
            fLocation = fThread.pyStringEval(fUnevalLocation);

            if (!fName.equals("")) 
            {
                evalElem = "process";
                evalAttr = "name";
                fName = fThread.pyStringEval(fUnevalName);
            }
            else
            {
                fName = "Process" + 
                        String.valueOf(fThread.getJob().getNextProcNumber());
                fUnevalName = fName;
            }

            evalAttr = "";
            fCommand = fThread.pyStringEval(fUnevalCommand);                

            if (!fCommandMode.equals(""))
            {
                evalElem = "command";
                evalAttr = "mode";
                fCommandMode= fThread.pyStringEval(fCommandMode);
                   
                if (!(fCommandMode.equalsIgnoreCase("shell")) &&
                    !(fCommandMode.equalsIgnoreCase("default")))
                {
                    // Raise a StartProcessError signal
                    fState = COMPLETE;
                    fThread.popAction();
                                     
                    String msg = "<process>\n" +
                        "\n  Name        : " + fName +
                        "\n  Location    : " + fLocation +
                        "\n  Command     : " + fCommand +
                        "\n  Command Mode: " + fCommandMode +
                        "\n\n  Invalid value for command element's mode " +
                        "attribute: " + fCommandMode +
                        "\n  Must be 'default' or 'shell'.";

                    fThread.setSignalMsgVar("STAXProcessStartErrorMsg", msg);
                    fThread.raiseSignal("STAXProcessStartError");

                    return 1;
                }

                if (fCommandMode.equalsIgnoreCase("shell"))
                {
                    request.append("SHELL ");

                    if (!fCommandShell.equals(""))
                    {
                        evalElem = "command";
                        evalAttr = "shell";
                        fCommandShell= fThread.pyStringEval(fCommandShell);
                        request.append(STAFUtil.wrapData(fCommandShell));
                        request.append(" ");
                    }
                }
                else
                    fCommandShell = "";
            }

            request.append("COMMAND ");
            request.append(STAFUtil.wrapData(fCommand));

            if (!fWorkload.equals("")) 
            {
                evalElem = "workload";
                evalAttr = "if";

                if (fThread.pyBoolEval(fWorkloadIf))
                {
                    evalAttr = "";
                    fWorkload = fThread.pyStringEval(fUnevalWorkload);
                    request.append(" WORKLOAD ");
                    request.append(STAFUtil.wrapData(fWorkload));
                }
                else
                    fWorkload = "";
            }

            if (!fTitle.equals(""))
            {
                evalElem = "title";
                evalAttr = "if";

                if (fThread.pyBoolEval(fTitleIf)) 
                {
                    evalAttr = "";
                    fTitle = fThread.pyStringEval(fUnevalTitle);
                    request.append(" TITLE ");
                    request.append(STAFUtil.wrapData(fTitle));
                }
                else 
                    fTitle = "";
            }
              
            if (!fParms.equals(""))
            {
                evalElem = "parms";
                evalAttr = "if";

                if (fThread.pyBoolEval(fParmsIf))
                {
                    evalAttr = "";
                    fParms = fThread.pyStringEval(fUnevalParms);
                    request.append(" PARMS ");
                    request.append(STAFUtil.wrapData(fParms));
                }
                else
                    fParms = "";
            } 
             
            if (!fWorkdir.equals(""))
            {
                evalElem = "workdir";
                evalAttr = "if";

                if (fThread.pyBoolEval(fWorkdirIf)) 
                {
                    evalAttr = "";
                    fWorkdir = fThread.pyStringEval(fUnevalWorkdir);
                    request.append(" WORKDIR ");
                    request.append(STAFUtil.wrapData(fWorkdir));
                }
                else
                    fWorkdir = "";
            }
            
            evalElem = "var";

            for (int i = 0; i < fUnevalVars.size(); i++)
            {
                evalAttr = "if";
                String varsIf = fThread.pyStringEval(
                    (String)fVarsIf.elementAt(i));

                if (fThread.pyBoolEval(varsIf))
                {
                    // Evaluate as a list; each item in list is a single var
                    evalAttr = "";
                    List varList = fThread.pyListEval(
                        (String)fUnevalVars.elementAt(i));
                    for (int j = 0; j < varList.size(); j++)
                    {
                        String var = ((Object)varList.get(j)).toString();
                        fVars.add(var);
                        request.append(" VAR ");
                        request.append(STAFUtil.wrapData(var));
                    }
                }
            }
            
            evalElem = "env";
            for (int i = 0; i < fUnevalEnvs.size(); i++)
            {
                evalAttr = "if";
                String envsIf = fThread.pyStringEval(
                    (String)fEnvsIf.elementAt(i));

                if (fThread.pyBoolEval(envsIf))
                {
                    // Evaluate as a list; each item in list is a single var
                    evalAttr = "";
                    List envList = fThread.pyListEval(
                        (String)fUnevalEnvs.elementAt(i));
                    for (int j = 0; j < envList.size(); j++)
                    {
                        String env = ((Object)envList.get(j)).toString();
                        fEnvs.add(env);
                        request.append(" ENV ");
                        request.append(STAFUtil.wrapData(env));
                    }        
                }
            }
                
            if (!fUseprocessvars.equals(""))
            {
                evalElem = "useprocessvars";
                evalAttr = "if";

                if (fThread.pyBoolEval(fUseprocessvarsIf)) 
                    request.append(" USEPROCESSVARS");
                else
                    fUseprocessvars = "";
            }
               
            if (!fStopusing.equals(""))
            {
                evalElem = "stopusing";
                evalAttr = "if";

                if (fThread.pyBoolEval(fStopusingIf)) 
                {
                    evalAttr = "";
                    fStopusing = fThread.pyStringEval(fUnevalStopusing);
                    request.append(" STOPUSING ");
                    request.append(STAFUtil.wrapData(fStopusing));
                }
                else
                    fStopusing = ""; 
            }
                
            if (!fConsole.equals(""))
            {
                evalElem = "console";
                evalAttr = "if";

                if (fThread.pyBoolEval(fConsoleIf))
                {
                    evalAttr = "use";
                    fConsole = fThread.pyStringEval(fUnevalConsole);

                    if (fConsole.equalsIgnoreCase("new")) 
                        request.append(" NEWCONSOLE");
                    else if (fConsole.equalsIgnoreCase("same")) 
                        request.append(" SAMECONSOLE");
                    else
                    {
                        // Raise a StartProcessError signal
                        fState = COMPLETE;
                        fThread.popAction();

                        String msg = "<process>\n" +
                            "\n  Name       : " + fName +
                            "\n  Location   : " + fLocation +
                            "\n  Command    : " + fCommand;

                        if (!fParms.equals(""))
                            msg += "\n  Parms      : " + fParms;

                        msg += "\n  Use Console: " + fConsole +
                            "\n\n  Invalid value for console element's use " +
                            "attribute: " + fConsole +
                            "\n  Must be 'new' or 'same'.";

                        fThread.setSignalMsgVar("STAXProcessStartErrorMsg",
                                                msg);
                        fThread.raiseSignal("STAXProcessStartError");

                        return 1;
                    }
                }
                else
                    fConsole = "";
            }

            if (!fFocus.equals(""))
            {
                evalElem = "focus";
                evalAttr = "if";

                if (fThread.pyBoolEval(fFocusIf))
                {
                    evalAttr = "mode";
                    fFocus = fThread.pyStringEval(fUnevalFocus);

                    if (fFocus.equalsIgnoreCase("minimized"))
                    {
                        fFocus = "Minimized";
                        request.append(" FOCUS Minimized");
                    }
                    else if (fFocus.equalsIgnoreCase("foreground"))
                    {
                        fFocus = "Foreground";
                        request.append(" FOCUS Foreground");
                    }
                    else if (fFocus.equalsIgnoreCase("background")) 
                    {
                        fFocus = "Background";
                        request.append(" FOCUS Background");
                    }
                    else
                    {
                        // Raise a StartProcessError signal
                        fState = COMPLETE;
                        fThread.popAction();

                        String msg = "<process>\n" +
                            "\n  Name       : " + fName +
                            "\n  Location   : " + fLocation +
                            "\n  Command    : " + fCommand;

                        if (!fParms.equals(""))
                            msg += "\n  Parms      : " + fParms;

                        msg += "\n  Focus Mode : " + fFocus +
                            "\n\n  Invalid value for focus element's mode " +
                            "attribute: " + fFocus +
                            "\n  Must be 'minimized', 'foreground' or " +
                            "'background'.";

                        fThread.setSignalMsgVar("STAXProcessStartErrorMsg",
                                                msg);
                        fThread.raiseSignal("STAXProcessStartError");

                        return 1;
                    }
                }
                else
                    fFocus = "";
            }
                
            if (!fUsername.equals(""))
            {
                evalElem = "username";
                evalAttr = "if";

                if (fThread.pyBoolEval(fUsernameIf))
                {
                    evalAttr = "";
                    fUsername = fThread.pyStringEval(fUnevalUsername);
                    request.append(" USERNAME ");
                    request.append(STAFUtil.wrapData(fUsername));
                }
                else
                    fUsername = "";
            }

            if (!fPassword.equals(""))
            {
                evalElem = "password";
                evalAttr = "if";

                if (fThread.pyBoolEval(fPasswordIf))
                {
                    evalAttr = "";
                    fPassword = fThread.pyStringEval(fUnevalPassword);
                    request.append(" PASSWORD ");
                    request.append(STAFUtil.wrapData(fPassword));
                }
                else
                    fPassword = "";
            }
            
            if (!fDisabledauth.equals(""))
            {
                evalElem = "disabledauth";
                evalAttr = "if";

                if (fThread.pyBoolEval(fDisabledauthIf))
                {
                    evalAttr = "action";

                    fDisabledauth = fThread.pyStringEval(fUnevalDisabledauth);

                    if (fDisabledauth.equalsIgnoreCase("error")) 
                        request.append(" DISABLEDAUTHISERROR");
                    else if (fDisabledauth.equalsIgnoreCase("ignore")) 
                        request.append(" IGNOREDISABLEDAUTH");
                    else
                    {
                        // Raise a StartProcessError signal
                        fState = COMPLETE;
                        fThread.popAction();
 
                        String msg = "<process>\n" +
                            "\n  Name                : " + fName +
                            "\n  Location            : " + fLocation +
                            "\n  Command             : " + fCommand;

                        if (!fParms.equals(""))
                            msg += "\n  Parms      : " + fParms;

                        msg += "\n  Disabled Auth Action: " + fDisabledauth +
                            "\n\n  Invalid value for disabledauth element's " +
                            "action attribute: " + fDisabledauth +
                            "\n  Must be 'error' or 'ignore'.";

                        fThread.setSignalMsgVar("STAXProcessStartErrorMsg",
                                                msg);
                        fThread.raiseSignal("STAXProcessStartError");

                        return 1;
                    }
                }
                else
                    fDisabledauth = "";
            }
                
            if (!fStatichandlename.equals(""))
            {
                evalElem = "statichandlename";
                evalAttr = "if";

                if (fThread.pyBoolEval(fStatichandlenameIf))
                {
                    evalAttr = "";
                    fStatichandlename = fThread.pyStringEval(
                                        fUnevalStatichandlename);
                    request.append(" STATICHANDLENAME ");
                    request.append(STAFUtil.wrapData(fStatichandlename));
                }
                else
                    fStatichandlename = "";
            }
                
            if (!fStdin.equals(""))
            {
                evalElem = "stdin";
                evalAttr = "if";

                if (fThread.pyBoolEval(fStdinIf))
                {
                    evalAttr = "";
                    fStdin = fThread.pyStringEval(fUnevalStdin);
                    request.append(" STDIN ");
                    request.append(STAFUtil.wrapData(fStdin));
                }
                else
                    fStdin = "";
            }
                
            if (!fStdout.equals(""))
            {
                evalElem = "stdout";
                evalAttr = "if";

                if (fThread.pyBoolEval(fStdoutIf))
                {
                    evalAttr = "mode";
                    fStdoutMode = fThread.pyStringEval(fUnevalStdoutMode);
 
                    evalAttr = "";
                    if (fStdoutMode.equalsIgnoreCase("replace"))
                    {
                        fStdout = fThread.pyStringEval(fUnevalStdout);
                        request.append(" STDOUT ");
                        request.append(STAFUtil.wrapData(fStdout));
                    }
                    else if (fStdoutMode.equalsIgnoreCase("append"))
                    {
                        fStdout = fThread.pyStringEval(fUnevalStdout);
                        request.append(" STDOUTAPPEND ");
                        request.append(STAFUtil.wrapData(fStdout));
                    } 
                    else
                    {
                        // Raise a StartProcessError signal
                        fState = COMPLETE;
                        fThread.popAction();
 
                        String msg = "<process>\n" +
                            "\n  Name       : " + fName +
                            "\n  Location   : " + fLocation +
                            "\n  Command    : " + fCommand;

                        if (!fParms.equals(""))
                            msg += "\n  Parms      : " + fParms;

                        msg += "\n  Stdout Mode: " + fStdoutMode +
                            "\n\n  Invalid value for stdout element's mode " +
                            "attribute: " + fStdoutMode +
                            "\n  Must be 'replace' or 'append'.";

                        fThread.setSignalMsgVar("STAXProcessStartErrorMsg",
                                                msg);
                        fThread.raiseSignal("STAXProcessStartError");
                        return 1;        

                    }
                }
                else
                {
                    fStdout = "";
                    fStdoutMode = "";
                }
            }
                
            if (!fStderr.equals("") || !fUnevalStderrMode.equals(""))
            {
                evalElem = "stderr";
                evalAttr = "if";

                if (fThread.pyBoolEval(fStderrIf)) 
                {
                    evalAttr = "mode";
                    fStderrMode = fThread.pyStringEval(fUnevalStderrMode);
                    evalAttr = "";

                    if (! fStderrMode.equalsIgnoreCase("stdout"))
                    {
                        // Check if EXACT same file name was specified
                        // for stdout and stderr and if so, set mode=stdout
                        fStderr = fThread.pyStringEval(fUnevalStderr);
                        if (fStderr.equals(fStdout))
                            fStderrMode = "stdout";
                    }
                        
                    if (fStderrMode.equalsIgnoreCase("replace"))
                    {
                        request.append(" STDERR ");
                        request.append(STAFUtil.wrapData(fStderr));
                    }                
                    else if (fStderrMode.equalsIgnoreCase("append"))
                    {
                        request.append(" STDERRAPPEND ");
                        request.append(STAFUtil.wrapData(fStderr));
                    }
                    else if (fStderrMode.equalsIgnoreCase("stdout"))
                    {
                        fStderr = "";
                        request.append(" STDERRTOSTDOUT");
                    }
                    else
                    {
                        // Raise a StartProcessError signal
                        fState = COMPLETE;
                        fThread.popAction();
 
                        String msg = "<process>\n" +
                            "\n  Name       : " + fName +
                            "\n  Location   : " + fLocation +
                            "\n  Command    : " + fCommand;

                        if (!fParms.equals(""))
                            msg += "\n  Parms      : " + fParms;

                        msg += "\n  Stderr Mode: " + fStderrMode +
                            "\n\n  Invalid value for stderr element's mode " +
                            "attribute: " + fStderrMode +
                            "\n  Must be 'replace' or 'append' or 'stdout'.";

                        fThread.setSignalMsgVar("STAXProcessStartErrorMsg",
                                                msg);
                        fThread.raiseSignal("STAXProcessStartError");
                        
                        return 1;
                    }
                }
                else
                {
                    fStderr = "";
                    fStderrMode = "";
                }
            }
            
            if (!fReturnStdout.equals(""))
            {
                evalElem = "returnstdout";
                evalAttr = "if";

                if (fThread.pyBoolEval(fReturnStdoutIf)) 
                    request.append(" RETURNSTDOUT");
                else
                    fReturnStdout = "";
            }
             
            if (!fReturnStderr.equals(""))
            {
                evalElem = "returnstderr";
                evalAttr = "if";

                if (fThread.pyBoolEval(fReturnStderrIf)) 
                    request.append(" RETURNSTDERR");
                else
                    fReturnStderr = "";
            }
                
            evalElem = "returnfile";

            for (int i = 0; i < fUnevalReturnFiles.size(); i++)
            {
                evalAttr = "if";
                String returnFilesIf = fThread.pyStringEval(
                    (String)fReturnFilesIf.elementAt(i));

                if (fThread.pyBoolEval(returnFilesIf))
                {
                    // Evaluate as a list; each list item is a single file
                        
                    evalAttr = "";
                    List fileList = fThread.pyListEval(
                        (String)fUnevalReturnFiles.elementAt(i));

                    for (int j = 0; j < fileList.size(); j++)
                    {
                        String file = ((Object)fileList.get(j)).toString();
                        fReturnFiles.add(file);
                        request.append(" RETURNFILE ");
                        request.append(STAFUtil.wrapData(file));
                    }
                }
            }

            if (fProcessAction != null)
            {
                evalElem = "process-action";
                evalAttr = "if";
                fProcessActionFlag = fThread.pyBoolEval(fProcessActionIf);
            }

            if (!fOther.equals(""))
            {
                evalElem = "other";
                evalAttr = "if";

                if (fThread.pyBoolEval(fOtherIf))
                {
                    fOther = fThread.pyStringEval(fUnevalOther);
                    request.append(" ");
                    request.append(fOther);
                }
                else
                    fOther = "";      
            }
        }
        catch (STAXPythonEvaluationException e)
        {
            fState = COMPLETE;
            fThread.popAction();

            String msg = "<process>\n" +
                "\n  Name    : " + fUnevalName +
                "\n  Location: " + fUnevalLocation +
                "\n  Command : " + fUnevalCommand;

            if (!fUnevalParms.equals(""))
                msg += "\n  Parms   : " + fUnevalParms;

            if (evalElem.equals("process"))
            {
                msg += "\n\n  Process 'name' attribute in error";
            }
            else
            {
                msg += "\n\n  Process sub-element in error: <" +
                    evalElem + ">";
                 
                if (!evalAttr.equals(""))
                {
                    msg += "\n  Attribute in error: " + evalAttr;
                }
            }

            fThread.setSignalMsgVar("STAXPythonEvalMsg", msg, e);
            fThread.raiseSignal("STAXPythonEvaluationError");

            return 1;
        }
                  
        fRequest = request.toString();

        return 0;
    }


    private void generateProcessStartEvent()
    {
        // Generate a start process event

        HashMap processMap = new HashMap();
        processMap.put("type", "process");
        processMap.put("block", fCurrentBlockName);
        processMap.put("status", "start");
        processMap.put("location", fLocation);
        processMap.put("command", STAFUtil.maskPrivateData(fCommand));
        processMap.put("handle", String.valueOf(fProcessHandle));
        processMap.put("parms", STAFUtil.maskPrivateData(fParms));
        processMap.put("name", fName);
        processMap.put("mode", fCommandMode);
        processMap.put("workload", fWorkload);
        processMap.put("title", fTitle);
        processMap.put("workdir", fWorkdir);

        if (!fCommandShell.equals(""))
        {    
            processMap.put("shell", fCommandShell);
        }

        // Convert the fVars vector to a list

        List varList = new ArrayList();

        for (int i = 0; i < fVars.size(); i++)
        {
            varList.add((String)fVars.elementAt(i));
        }

        processMap.put("varList", STAFMarshallingContext.marshall(
            varList, new STAFMarshallingContext()));

        // Convert the fEnvs vector to a list

        List envList = new ArrayList();

        for (int i = 0; i < fEnvs.size(); i++)
        {
            envList.add((String)fEnvs.elementAt(i));
        }

        processMap.put("envList", STAFMarshallingContext.marshall(
            envList, new STAFMarshallingContext()));
        
        if (!fUseprocessvars.equals(""))
        {
            processMap.put("useprocessvars", "");
        }

        processMap.put("stopusing", fStopusing);

        if (fConsole.equalsIgnoreCase("new"))
        {
            processMap.put("newconsole", "");
        }
        else if (fConsole.equalsIgnoreCase("same"))
        {
            processMap.put("sameconsole", "");
        }

        if (!fFocus.equals(""))
        {
            processMap.put("focus", fFocus);
        }

        processMap.put("username", fUsername);

        if (!fPassword.equals(""))
        {
            processMap.put("password", "*******");
        }

        if (fDisabledauth.equalsIgnoreCase("error"))
        {
            processMap.put("disabledauthiserror", "");
        }
        else if (fDisabledauth.equalsIgnoreCase("ignore"))
        {
            processMap.put("ignoredisabledauth", "");
        }

        processMap.put("stdin", fStdin);       

        if (fStdoutMode.equalsIgnoreCase("replace"))
        {
            processMap.put("stdout", fStdout);
        }
        else if (fStdoutMode.equalsIgnoreCase("append"))
        {
            processMap.put("stdoutappend", fStdout);
        }

        if (fStderrMode.equalsIgnoreCase("replace"))
        {
            processMap.put("stderr", fStderr);
        }
        else if (fStderrMode.equalsIgnoreCase("append"))
        {
           processMap.put("stderrappend", fStderr);
        }
        else if (fStderrMode.equalsIgnoreCase("stdout"))
        {
            processMap.put("stderrtostdout", "");
        }

        if (!fReturnStdout.equals(""))
        {
            processMap.put("returnstdout", "");
        }

        if (!fReturnStderr.equals(""))
        {
            processMap.put("returnstderr", "");
        }

        // Convert the fReturnFiles vector to a string containing a marshalled
        // list of strings

        List returnFileList = new ArrayList();

        for (int i = 0; i < fReturnFiles.size(); i++)
        {
            returnFileList.add((String)fReturnFiles.elementAt(i));
        }

        processMap.put("returnFileList", STAFMarshallingContext.marshall(
            returnFileList, new STAFMarshallingContext()));
        
        processMap.put("statichandlename", fStatichandlename);

        processMap.put("other", fOther);

        fThread.getJob().generateEvent(
            STAXProcessActionFactory.STAX_PROCESS_EVENT, processMap);
    }
    

    private void generateProcessStopEvent()
    {
        // Generate a stop process event

        HashMap processMap = new HashMap();
        processMap.put("type", "process");
        processMap.put("block", fCurrentBlockName);
        processMap.put("status", "stop");
        processMap.put("location", fLocation);
        processMap.put("command", STAFUtil.maskPrivateData(fCommand));
        processMap.put("handle", String.valueOf(fProcessHandle));
        processMap.put("parms", STAFUtil.maskPrivateData(fParms));
        processMap.put("name", fName);

        fThread.getJob().generateEvent(
            STAXProcessActionFactory.STAX_PROCESS_EVENT, processMap);
    }


    private String replace(String source, String find, String replace) 
    {
        StringBuffer result = new StringBuffer(source);
        int position = source.indexOf(find);
        
        while (position > -1) 
        {
            result.replace(position, position + find.length(), replace);
            position = result.toString().indexOf(find, position +
            replace.length());
        }
        
        return result.toString();
    }

    STAXThread fThread = null;
    int fState = INIT;
    
    private String fLocation = new String();
    private String fCommand = new String();
    private String fName = new String();
    private String fWorkload = new String("");
    private String fWorkloadIf = new String("");
    private String fTitle = new String("");
    private String fTitleIf = new String("");
    private String fParms = new String("");
    private String fParmsIf = new String("");
    private String fWorkdir = new String("");
    private String fWorkdirIf = new String("");
    private Vector fVars = new Vector();
    private Vector fVarsIf = new Vector();
    private Vector fEnvs = new Vector();
    private Vector fEnvsIf = new Vector();
    private String fUseprocessvars = new String("");
    private String fUseprocessvarsIf = new String("");
    private String fStopusing = new String("");
    private String fStopusingIf = new String("");
    private String fConsole = new String("");
    private String fConsoleIf = new String("");
    private String fFocus = new String("");
    private String fFocusIf = new String("");
    private String fUsername = new String("");
    private String fUsernameIf = new String("");
    private String fPassword = new String("");
    private String fPasswordIf = new String("");
    private String fDisabledauth = new String("");
    private String fDisabledauthIf = new String("");
    private String fStatichandlename = new String("");
    private String fStatichandlenameIf = new String("");
    private String fStdin = new String("");
    private String fStdinIf = new String("");
    private String fStdout = new String("");
    private String fStdoutIf = new String("");
    private String fStdoutMode = new String("");
    private String fStderr = new String("");
    private String fStderrIf = new String("");
    private String fStderrMode = new String("");
    private String fReturnStdout = new String("");
    private String fReturnStdoutIf = new String("");
    private String fReturnStderr = new String("");
    private String fReturnStderrIf = new String("");
    private Vector fReturnFiles = new Vector();
    private Vector fReturnFilesIf = new Vector();
    private String fOther = new String("");
    private String fOtherIf = new String("");
    private String fCommandMode = new String("");
    private String fCommandShell = new String("");
    private STAXAction fProcessAction = null;
    private String fProcessActionIf = new String("");
    
    private String fUnevalLocation = new String();
    private String fUnevalCommand = new String();
    private String fUnevalName = new String();
    private String fUnevalWorkload = new String("");
    private String fUnevalTitle = new String("");
    private String fUnevalParms = new String("");
    private String fUnevalWorkdir = new String("");
    private Vector fUnevalVars = new Vector();
    private Vector fUnevalEnvs = new Vector();
    private String fUnevalUseprocessvars = new String("");
    private String fUnevalStopusing = new String("");
    private String fUnevalConsole = new String("");
    private String fUnevalFocus = new String("");
    private String fUnevalUsername = new String("");
    private String fUnevalPassword = new String("");
    private String fUnevalDisabledauth = new String("");
    private String fUnevalStatichandlename = new String("");
    private String fUnevalStdin = new String("");
    private String fUnevalStdout = new String("");
    private String fUnevalStdoutMode = new String("");
    private String fUnevalStderr = new String("");
    private String fUnevalStderrMode = new String("");
    private String fUnevalReturnStdout = new String("");
    private String fUnevalReturnStderr = new String("");
    private Vector fUnevalReturnFiles = new Vector();
    private String fUnevalOther = new String("");
    private String fUnevalProcessActionIf = new String("");
    private String fUnevalCommandMode = new String("");
    private String fUnevalCommandShell = new String("");

    private STAXHoldThreadCondition fHoldCondition = 
        new STAXHoldThreadCondition("Process");
    private int fRequestNumber = 0;
    private int fRequestRC = 0;
    private String fRequest = new String();
    private String fRequestResult = new String();
    private int fProcessHandle = 0;
    private long fProcessRC = -1;
    private String fProcessTimestamp = new String();
    private STAXTimestamp fStartTimestamp;
    private STAXProcessActionFactory fFactory;
    private String fCurrentBlockName = new String();
    private HashMap fThreadMap = new HashMap();
    private boolean fProcessActionFlag = true;
    private STAXTimedEvent fTimedEvent = null;
    private String fProcessKey = "";
    private boolean fUsingProcessKeyOption = true;
}