/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2002                                              */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

package com.ibm.staf.service.stax;
import java.util.TreeMap;

public class STAXRaiseAction implements STAXAction
{
    private STAXRaiseAction()
    { /* Do Nothing */ }

    public STAXRaiseAction(String signal)
    {
        fUnevalSignal = signal;
        fSignal = signal;
    }

    public String getXMLInfo()
    {
        return "<raise signal=\"" + fSignal + "\"/>";
    }

    public String getInfo()
    {
        return fSignal;
    }

    public String getDetails()
    {
        return "Signal:" + fSignal;
    }

    public void execute(STAXThread thread)
    {
        thread.popAction();

        try
        {
            fSignal = thread.pyStringEval(fUnevalSignal);
        }
        catch (STAXPythonEvaluationException e)
        {
            thread.setSignalMsgVar("STAXPythonEvalMsg", getXMLInfo(), e);
            thread.raiseSignal("STAXPythonEvaluationError");
            return;
        }

        thread.raiseSignal(fSignal);
    }

    public void handleCondition(STAXThread thread, STAXCondition cond)
    {
        thread.popAction();
    }

    public STAXAction cloneAction()
    {
        STAXRaiseAction clone = new STAXRaiseAction();

        clone.fUnevalSignal = fUnevalSignal;
        clone.fSignal = fSignal;

        return clone;
    }

    private String fUnevalSignal;
    private String fSignal;
}
