/*****************************************************************************/
/* Software Testing Automation Framework (STAF)                              */
/* (C) Copyright IBM Corp. 2002                                              */
/*                                                                           */
/* This software is licensed under the Common Public License (CPL) V1.0.     */
/*****************************************************************************/

package com.ibm.staf.service.stax;
import java.util.TreeMap;

public class STAXTestcaseStatusAction implements STAXAction
{
    private STAXTestcaseStatusAction()
    { /* Do Nothing */ }

    public STAXTestcaseStatusAction(String status, String message)
    {
        fUnevalStatus = status;
        fStatus = status;        
        fUnevalMessage = message;
        fMessage = message;
    }

    public String getXMLInfo()
    {
        String msg = "<tcstatus result=\"";
        msg += fStatus;        
        msg += "\">" + fMessage + "</tcstatus>";

        return msg;
    }

    public String getInfo()
    {
        return fStatus;        
    }

    public String getDetails()
    {
        return "Status:" + fStatus + 
               ";Message:" + fMessage;
    }

    public void execute(STAXThread thread)
    {
        thread.popAction();

        try
        {
            if (fUnevalStatus == null)
                fStatus = new String();
            else
                fStatus = thread.pyStringEval(fUnevalStatus);        

            if (fUnevalMessage == null)
                fMessage = new String();
            else
                fMessage = thread.pyStringEval(fUnevalMessage);
        }
        catch (STAXPythonEvaluationException e)
        {
            thread.setSignalMsgVar("STAXPythonEvalMsg", getXMLInfo(), e);
            thread.raiseSignal("STAXPythonEvaluationError");
            return;
        }

        // Get Current Testcase

        String testcaseName;
        try
        {
            testcaseName = thread.pyStringEval("STAXCurrentTestcase");
            if (testcaseName.equals("None"))
                throw new STAXPythonEvaluationException("");
        }
        catch (STAXPythonEvaluationException e)
        {
            // Raise a signal if no testcase wrapper element for tcstatus.

            thread.setSignalMsgVar("STAXTestcaseMissingMsg", getXMLInfo());
            thread.raiseSignal("STAXTestcaseMissingError");
            return;
        }

        // Raise a signal if status result is not "pass" or "fail".

        if (!fStatus.equalsIgnoreCase("pass") &&
            !fStatus.equalsIgnoreCase("fail"))
        {
            String msg = getXMLInfo() + "\n\n  Testcase name: " + testcaseName;
            thread.setSignalMsgVar("STAXInvalidTcStatusResultMsg", msg);
            thread.raiseSignal("STAXInvalidTcStatusResult");
            return;
        } 

        // Update Testcase Status

        TreeMap testcaseMap = (TreeMap)thread.getJob().getData("testcaseMap");

        synchronized (testcaseMap)
        {
            STAXTestcase theTest = (STAXTestcase)testcaseMap.get(testcaseName);

            if (theTest == null)
            {
                theTest = new STAXTestcase(testcaseName);
                testcaseMap.put(testcaseName, theTest);
            }

            theTest.updateStatus(fStatus, fMessage, thread.getJob());
        }
    }

    public void handleCondition(STAXThread thread, STAXCondition cond)
    {
        thread.popAction();
    }

    public STAXAction cloneAction()
    {
        STAXTestcaseStatusAction clone = new STAXTestcaseStatusAction();

        clone.fUnevalStatus = fUnevalStatus;
        clone.fStatus = fStatus;
        clone.fUnevalMessage = fUnevalMessage;
        clone.fMessage = fMessage;

        return clone;
    }

    private String fUnevalStatus;
    private String fStatus;
    private String fUnevalMessage;
    private String fMessage;
}
